from audiostack.content.script import Script
from audiostack.content.media import Media
from audiostack.content.file import File, Folder
from audiostack.content.recommend import RecommendTag, RecommendMood, RecommendTone


def list_projects():
    from audiostack.content.root_functions import Root

    return Root.list_projects()


def list_modules():
    from audiostack.content.root_functions import Root

    return Root.list_modules()

def generate(prompt: str, max_length: int = 100):
    from audiostack.content.root_functions import Root

    return Root.generate(prompt, max_length)


from audiostack.helpers.request_interface import RequestInterface
from audiostack.helpers.request_types import RequestTypes
from audiostack.helpers.api_item import APIResponseItem
from audiostack.content.file import File
from typing import List, Optional

class Suite:
    DENOISE_ENDPOINT = "suite/denoise"
    class FailedPipeline(Exception):
        pass
    
    interface = RequestInterface(family="production")

    class EvaluationItem(APIResponseItem):
        def __init__(self, response) -> None:
            super().__init__(response)

    class PipelineInProgressItem(APIResponseItem):
        def __init__(self, response) -> None:
            super().__init__(response)
            
            self.pipelineId = self.data["pipelineId"]

    class PipelineFinishedItem(PipelineInProgressItem):
        def __init__(self, response) -> None:
            super().__init__(response)
            self.newFileIds = self.data["results"]["newFileIds"]
            self.inputFileIds = self.data["results"]["inputFileIds"]
            self.replacedFileIds = self.data["results"]["replacedFileIds"]
        
        def convert_new_files_to_items(self) -> List["File.Item"]:
            return [File.get(f["fileId"]) for f in self.newFileIds]
        
        def convert_replaced_files_to_items(self) -> List["File.Item"]:
            return [File.get(f["fileId"]) for f in self.replacedFileIds]
        
    @staticmethod
    def evaluate(
        fileId: str,
        preset: str = "",
        processes: list = [],
        text: str = "",
        scriptId: str = "",
        language: str = "en-US",
    ) -> EvaluationItem:
        if not (fileId):
            raise Exception("fileId should be supplied")
        if text and scriptId:
            raise Exception("either text or scriptId or none should be supplied not both")
        if not isinstance(processes, list):
            raise Exception("processes should be a list")
        if not isinstance(preset, str):
            raise Exception("preset should be a string")

        body = {
            "fileId": fileId,
            "preset": preset,
            "processes": processes,
            "language": language
        }
        if text:
            body["text"] = text
        elif scriptId:
            body["scriptId"] = scriptId
        
        r = Suite.interface.send_request(rtype=RequestTypes.POST, route="suite/evaluate", json=body)

        while r["statusCode"] != 200 and r["statusCode"] !=404:   # TODO REVISE
            print("Response in progress please wait...")
            r = Suite.interface.send_request(rtype=RequestTypes.POST, route="suite/evaluate", json=body)
            
        return Suite.EvaluationItem(r)
    
    @staticmethod
    def separate(fileId: str, wait=True):

        body = {
            "fileId": fileId,
        }
        r = Suite.interface.send_request(rtype=RequestTypes.POST, route="suite/separate", json=body)
        item = Suite.PipelineInProgressItem(r)
        return Suite._poll(r, item.pipelineId) if wait else item
    
    @staticmethod
    def denoise(fileId: str, level: Optional[int] = None, wait=True):

        body = {
            "fileId": fileId,
            "level": level
        }
        r = Suite.interface.send_request(rtype=RequestTypes.POST, route=Suite.DENOISE_ENDPOINT, json=body)
        item = Suite.PipelineInProgressItem(r)
        return Suite._poll(r, item.pipelineId) if wait else item
        

    @staticmethod
    def _poll(r, pipelineId: str):
        while r["statusCode"] == 202:
            r = Suite.interface.send_request(
                rtype=RequestTypes.GET, route="suite/pipeline", path_parameters=pipelineId
            )
        
        status = r.get("data", {}).get("status", 200)
        if status > 400:
            msg = r.get("data", {}).get("message")
            errors = r.get("data", {}).get("errors")
            raise Suite.FailedPipeline("pipeline failed: ", msg, "errors are as follows: ", ','.join(errors))
        
        return Suite.PipelineFinishedItem(r)


    @staticmethod
    def get(pipelineId: str):
        r = Suite.interface.send_request(rtype=RequestTypes.GET, route="suite/pipeline", path_parameters=pipelineId)
        
        return Suite._poll(r, pipelineId)


sdk_version = "1.3.1"
api_base = "https://v2.api.audio"
api_key = None
assume_org_id = None
app_info = None


from warnings import warn

from audiostack import content as Content
from audiostack import speech as Speech
from audiostack import production as Production
from audiostack import delivery as Delivery
from audiostack.docs.docs import Documentation

warn(
    "Future releases (`^2.0.0`) of the AudioStack SDK will only support python `^3.8.1`",
    DeprecationWarning,
    stacklevel=2,
)


billing_session = 0


def credits_used_in_this_session():
    return float("{:.2f}".format(billing_session))



from audiostack.helpers.request_interface import RequestInterface
from audiostack.helpers.request_types import RequestTypes
from audiostack.helpers.api_item import APIResponseItem
from audiostack.helpers.api_list import APIResponseList


class Mix:
    interface = RequestInterface(family="production")

    class Item(APIResponseItem):
        def __init__(self, response) -> None:
            super().__init__(response)
            self.productionId = self.data["productionId"]

        def download(self, fileName="", path="./") -> None:
            sections = self.data["files"]
            for i, s in enumerate(sections):
                format = s["format"]
                original_name = s["filename"]

                if not fileName:
                    full_name = f"{original_name}.{format}"
                else:
                    full_name = f"{fileName}_{i}.{format}"

                RequestInterface.download_url(
                    s["url"], destination=path, name=full_name
                )

        def delete(self):
            return Mix.delete(self.productionId)

    class List(APIResponseList):
        def __init__(self, response, list_type) -> None:
            super().__init__(response, list_type)

        def resolve_item(self, list_type, item):
            if list_type == "productionIds":
                return Mix.Item({"data": item})
            elif list_type == "presets":
                return

            else:
                raise Exception()

    @staticmethod
    def create(
        speechId: str="",
        scriptId: str="",
        speechItem=None,
        soundTemplate: str = "",
        mediaFiles: dict = {},
        fxFiles: dict = {},
        sectionProperties: dict = {},
        timelineProperties: dict = {},
        masteringPreset: str = "",
        public: bool = False,
        exportSettings: dict = {},
        strictValidation: bool = True,
        validate: bool = False,
        soundLayer: str = "default"
    ) -> Item:
        counts = sum([1 for i in [speechId, scriptId, speechItem] if i])
        if counts != 1:
            raise Exception("only 1 of the following is required; speechId, speechItem, or scriptId")
        
        if speechItem:
            speechId = speechItem.speechId

        if not isinstance(soundTemplate, str):
            raise Exception("soundTemplate argument should be a string")
        if not isinstance(masteringPreset, str):
            raise Exception("masteringPreset should be a string")
        

        body = {
            "soundTemplate": soundTemplate,
            "mediaFiles": mediaFiles,
            "fxFiles": fxFiles,
            "sectionProperties": sectionProperties,
            "timelineProperties": timelineProperties,
            "soundLayer": soundLayer,
            "masteringPreset": masteringPreset,
            "public": public,
            "exportSettings" : exportSettings,
            "strictValidation" : strictValidation
        }
        if speechId:
            body["speechId"] = speechId
        elif scriptId:
            body["scriptId"] = scriptId

        if validate:
            r = Mix.interface.send_request(rtype=RequestTypes.POST, route="validate", json=body)
        else:
            r = Mix.interface.send_request(rtype=RequestTypes.POST, route="mix", json=body)
            
        while r["statusCode"] == 202:
            print("Response in progress please wait...")
            r = Mix.interface.send_request(
                rtype=RequestTypes.GET, route="mix", path_parameters=r["data"]["productionId"]
            )
        
        return Mix.Item(r)

    @staticmethod
    def get(productionId: str) -> Item:
        r = Mix.interface.send_request(
            rtype=RequestTypes.GET, route="mix", path_parameters=productionId
        )
        return Mix.Item(r)

    @staticmethod
    def delete(productionId: str) -> str:
        r = Mix.interface.send_request(
            rtype=RequestTypes.DELETE, route="mix", path_parameters=productionId
        )
        return APIResponseItem(r)

    @staticmethod
    def list(
        projectName="", moduleName: str = "", scriptName: str = "", scriptId: str = ""
    ) -> list:
        query_params = {
            "projectName": projectName,
            "moduleName": moduleName,
            "scriptName": scriptName,
            "scriptId": scriptId,
        }

        r = Mix.interface.send_request(
            rtype=RequestTypes.GET, route="mixes", query_parameters=query_params
        )
        return Mix.List(r, list_type="productionIds")

    @staticmethod
    def list_presets() -> Item:
        r = Mix.interface.send_request(
            rtype=RequestTypes.GET, route="mix/presets", path_parameters=""
        )
        return Mix.List(response=r, list_type="presets")



import os
import re
from setuptools import setup, find_packages

ROOT = os.path.dirname(__file__)
VERSION_RE = re.compile(r"""sdk_version = ['"]([0-9.]+)['"]""")

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


def get_version():
    init = open(os.path.join(ROOT, "audiostack", "__init__.py")).read()
    return VERSION_RE.search(init).group(1)


setup(
    name="audiostack",
    version=get_version(),
    author="Aflorithmic",
    author_email="sam@aflorithmic.ai",
    description="Python SDK for Audiostack API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/aflorithmic/audiostack-python",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=['requests >= 2.28.1; python_version >= "3.6"'],
    python_requires=">=3.6",
    py_modules=["audiostack"],
)
